@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2001 Bill Wilson
+|  Copyright (C) 1999-2002 Bill Wilson
 |
 |  Author:  Bill Wilson    bill@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -12,7 +12,8 @@
 |  This program is distributed in the hope that it will be useful,
 |  but WITHOUT ANY WARRANTY; without even the implied warranty of
 |  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.
+|  GNU General Public License for more details.  Version 2 is in the
+|  COPYRIGHT file in the top level directory of this distribution.
 | 
 |  To get a copy of the GNU General Puplic License, write to the Free Software
 |  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -52,8 +53,12 @@
 
 #include <locale.h>
 
+#if defined(__sun) && defined(__SVR4)
+#define __solaris__
+#endif
+
 #if !defined(__FreeBSD__) && !defined(__linux__) && !defined(__NetBSD__) \
-	&& !defined(__OpenBSD__)
+	&& !defined(__OpenBSD__) && !defined(__solaris__)
 #define  USE_LIBGTOP
 #endif
 
@@ -102,8 +107,8 @@
 /* -------------------------------------------------------------------
 */
 #define	GKRELLM_VERSION_MAJOR	1
-#define	GKRELLM_VERSION_MINOR	0
-#define	GKRELLM_VERSION_REV		7
+#define	GKRELLM_VERSION_MINOR	2
+#define	GKRELLM_VERSION_REV		9
 /* Backward compatibility */
 #define	VERSION_MAJOR			GKRELLM_VERSION_MAJOR
 #define	VERSION_MINOR			GKRELLM_VERSION_MINOR
@@ -111,13 +116,14 @@
 
 #define	GKRELLM_DIR				".gkrellm"
 #define	GKRELLM_USER_CONFIG		".gkrellm/user_config"
-#define	GKRELLM_THEME_CONFIG	".gkrellm/gkrellm_theme.cfg"
+#define	GKRELLM_THEME_CONFIG	".gkrellm/theme_config"
 #define GKRELLM_THEMES_DIR		".gkrellm/themes"
 #define	GKRELLM_DATA_DIR		".gkrellm/data"
 #define	GKRELLM_PLUGINS_DIR		".gkrellm/plugins"
 #define GKRELLMRC				"gkrellmrc"
 
-#define	PLUGINS_INSTALL_LIST	".gkrellm/plugins_install"
+#define	PLUGINS_INSTALL_FILE		".gkrellm/plugins_install"
+#define	PLUGINS_PLACEMENT_FILE	".gkrellm/plugins_placement"
 
 #define	LOCAL_THEMES_DIR		"/usr/local/share/gkrellm/themes"
 #if !defined(SYSTEM_THEMES_DIR)
@@ -138,9 +144,13 @@
 
   /* Label midpoints are positioned as a percent of chart_width.
   */
-#define	LABEL_NONE		-1
-#define	LABEL_CENTER	50
-#define LABEL_MAX		100
+#define	GKRELLM_LABEL_NONE		-1
+#define	GKRELLM_LABEL_CENTER	50
+#define GKRELLM_LABEL_MAX		100
+  /* Pre 1.2.9 compatibility defines */
+#define	LABEL_NONE		GKRELLM_LABEL_NONE
+#define	LABEL_CENTER	GKRELLM_LABEL_CENTER
+#define LABEL_MAX		GKRELLM_LABEL_MAX
 
 #define	GRID_MODE_NORMAL		0
 #define	GRID_MODE_RESTRAINED	1
@@ -206,7 +216,7 @@ typedef struct
 	GdkFont		*font;
 	GdkFont		**font_seed;
 	gint		effect;
-	gint		spare;
+	gint		internal;
 	GdkColor	color;
 	GdkColor	shadow_color;
 	}
@@ -216,7 +226,9 @@ typedef struct
 
   /* Values for Decal flags
   */
-#define DF_LOCALPIXMAPS		0x1
+#define DF_LOCAL_PIXMAPS	0x1
+#define DF_TOP_LAYER		0x2
+#define	DF_MOVED			0x4
 
   /* Values for Decal state
   */
@@ -243,6 +255,7 @@ typedef struct
 	gint		x_off,
 				y_baseline;
 	TextStyle	text_style;	/* Used if decal is a drawn text string		*/
+	gint		x_old, y_old;	/* Used if decal was moved */
 	}
 	Decal;
 
@@ -289,22 +302,36 @@ typedef struct
 #define	SET_KRELL_X_HOT			3
 #define	SET_KRELL_EMA_PERIOD	4
 #define	SET_KRELL_DEPTH			5
-#define	SET_LABEL_POSITION		6
-#define	SET_MARGIN				7
-#define	SET_TOP_MARGIN			8
-#define	SET_BOTTOM_MARGIN		9
-#define	SET_TEXTCOLOR_A			10
-#define	SET_TEXTCOLOR_B			11
-#define	SET_TEXTFONT_A			12
-#define	SET_TEXTFONT_B			13
-#define	SET_BORDER				14
-#define	SET_TRANSPARENCY		15
+#define	SET_KRELL_LEFT_MARGIN	6
+#define	SET_KRELL_RIGHT_MARGIN	7
+#define	SET_LABEL_POSITION		8
+#define	SET_MARGIN				9
+#define	SET_MARGINS				10
+#define	SET_LEFT_MARGIN			11
+#define	SET_RIGHT_MARGIN		12
+#define	SET_TOP_MARGIN			13
+#define	SET_BOTTOM_MARGIN		14
+#define	SET_TEXTCOLOR_A			15
+#define	SET_TEXTCOLOR_B			16
+#define	SET_TEXTFONT_A			17
+#define	SET_TEXTFONT_B			18
+#define	SET_BORDER				19
+#define	SET_TRANSPARENCY		20
 
 #define	TS_A		1
 #define	TS_B		2
 
 #define BIT(a)	(1 << (a))
 
+typedef struct
+	{
+	gint	left,
+			right,
+			top,
+			bottom;
+	}
+	Margin;
+
   /* Some of these style entries do not apply to all monitors.
   */
 typedef struct
@@ -318,9 +345,8 @@ typedef struct
 				krell_ema_period,
 				krell_depth,
 				krell_spare;
-
 	gint		label_position,
-				margin,			/* Text left/right margins in a panel */
+				margin,			/* 1.2.9 deprecated, use margins */
 				spare;
 	TextStyle	label_tsA;
 	TextStyle	label_tsB;
@@ -328,8 +354,10 @@ typedef struct
 	GdkImlibBorder
 				border;			/* Border for background panel image */
 	gint		transparency;
-	gint		top_margin,		/* Text top margin in a panel */
-				bottom_margin;	/* Text bottom margin in a panel */
+	gboolean	themed;			/* True if theme has custom assignments */
+	Margin		margins;
+	gint		krell_left_margin,
+				krell_right_margin;
 	}
 	Style;
 
@@ -341,10 +369,14 @@ typedef struct
   |  a maxed out GKrellM display, I call them Krells - inspired by the
   |  wall full of power monitoring meters the Krell had in Forbidden Planet.
   */
-  /* Krell.expand values (not sure I will keep these in) */
-#define	NONE	0
-#define	LEFT	1
-#define	RIGHT	2
+  /* Krell.expand values */
+#define	KRELL_EXPAND_NONE				0
+#define	KRELL_EXPAND_LEFT				1
+#define	KRELL_EXPAND_RIGHT				2
+#define	KRELL_EXPAND_BAR_MODE			3
+#define KRELL_EXPAND_LEFT_SCALED		4
+#define KRELL_EXPAND_RIGHT_SCALED		5
+#define	KRELL_EXPAND_BAR_MODE_SCALED	6
 
 typedef struct
 	{
@@ -361,7 +393,7 @@ typedef struct
 	gint		last_reading;
 	gint		full_scale;
 	gulong		previous;
-	gulong		previous_total;	/* For dynamic full_scale calc of % krells */
+	gulong		value;
 
 	gint		modified;	/* True if krell has moved.			*/
 
@@ -376,6 +408,8 @@ typedef struct
 	gint		ema;		/* exponential moving average */
 	gint		period;		/* of ema */
 	gint		full_scale_expand;
+	gboolean	monotonic;
+	gint		bar_mode;
 
 	Draw_rec	old_draw;	/* Last draw for restoring background.	*/
 	Draw_rec	draw;		/* Parameters to currently drawn krell	*/
@@ -386,8 +420,6 @@ typedef struct
   */
 #define	KRELL(p)	((Krell *)(((p)->krell)->data))
 
-
-
 typedef struct
 	{
 	GtkWidget	*hbox;			/* Container box this area is packed into */
@@ -402,11 +434,16 @@ typedef struct
 	GList		*decal;			/* Linked list of decals for this panel	*/
 	GList		*krell;			/* Linked list of krells for this panel	*/
 	Style		*style;
-	gint		modified;		/* True if a decal is modified.			*/
+	gint		modified;		/* True if decal/krell modified.		*/
 	gint		x, y, w, h;
-
 	GdkPixmap	*bg_clean_pixmap;	/* A no label bg_pixmap				*/
+
 	gint		transparency;
+	gboolean	keep_lists;
+	gboolean	shown;
+	gboolean	bg_image_override;
+	GdkImlibImage *bg_image;
+	gpointer	monitor;
 	}
 	Panel;
 
@@ -414,72 +451,190 @@ typedef struct
 #define CHART_WIDTH_MAX	1000
 #define CHART_WIDTH_MIN	25
 
+  /* Each chart must have a ChartConfig struct associated with it.
+  */
+#define	GKRELLM_CHARTCONFIG_KEYWORD	"chart_config"
+
+  /* Values for ChartConfig flags */
+#define	NO_CONFIG_AUTO_GRID_RESOLUTION	1
+#define	NO_CONFIG_FIXED_GRIDS			2
+
+typedef struct
+	{
+	gint		flags;
+	gboolean	config_loaded;
+	gboolean	log;
+
+	gint		h;
+	void		(*cb_height)();
+	gpointer	cb_height_data;
+	GtkWidget	*height_spin_button;
+
+	/* grid_resolution must be an integer and will be mapped into an integer
+	|  1,2,5 or 1,1.5,2,3,5,7 sequence if map_sequence is TRUE.  If
+	|  map_sequence is false, grid_resolution will be the spin_button
+	|  value * spin_factor so spin buttons may have non-integer settings.
+	*/
+	gint		grid_resolution;
+	gboolean	auto_grid_resolution;
+	gboolean	auto_resolution_stick;
+	gboolean	sequence_125;
+	void		(*cb_grid_resolution)();
+	gpointer	cb_grid_resolution_data;
+	GtkWidget	*grid_resolution_spin_button;
+	GtkWidget	*auto_resolution_control_menubar;
+	GtkItemFactory	*auto_resolution_item_factory;
+	gchar		*grid_resolution_label;
+	gboolean	adjustment_is_set;
+	gboolean	map_sequence;
+	gfloat		spin_factor;
+	gfloat		low,		/* Stuff for the grid_resolution spin adjustment */
+				high,
+				step0,
+				step1;
+	gint		digits,
+				width;
+
+	gboolean	fixed_grids;
+	void		(*cb_fixed_grids)();
+	gpointer	cb_fixed_grids_data;
+	GtkWidget	*fixed_grids_spin_button;
+
+	GList		*cd_list;			/* Same list as in parent Chart */
+	GList		**chart_cd_list;
+	}
+	ChartConfig;
+
+  /* Charts are drawn in layers and each data value drawn has its own layer
+  |  (the ChartData struct -> image/color of the drawn data and
+  |  the data colored grid lines).  Each layer is a chart sized pixmap
+  |  with grid lines drawn on it.  A chart is drawn by drawing the
+  |  background followed by stenciling each data layer through a
+  |  data_bitmap.
+  */
+typedef struct
+	{
+	GdkPixmap	*pixmap;			/* Working layer pixmap with grid lines	*/
+	GdkPixmap	**src_pixmap;		/* The layer source image/color		*/
+	GdkPixmap	*grid_pixmap;		/* The grid pixmap to draw on src 	*/
+	}
+	ChartLayer;
+
 typedef struct
 	{
-	gchar		*name;
 	GtkWidget	*hbox;			/* Container box this chart is packed into */
 	GtkWidget	*drawing_area;
-	GdkPixmap	*pixmap;			/* The expose pixmap.		*/
-
-	GdkPixmap	*bg_pixmap;			/* Background of chart		*/
-	GdkPixmap	*bg_grided_pixmap;	/* Background with drawn grid lines */
-	GdkPixmap	*grid_pixmap;		/* Grid to draw on bg_grided_pixmap*/
-
-	GdkPixmap	*in_grided_pixmap,	/* Data_in image with grid lines on it	*/
-				*out_grided_pixmap;	/* Data_out image with grid lines on it	*/
-	GdkBitmap	*in_data_bitmap,	/* Draw in_data here and use as stencil */
-				*out_data_bitmap;	/* Draw out_data here and use as stencil */
 
 	Panel		*panel;
-	gint		x, y, w, h;		/* Location inside monitor */
-	gint		private;
-	gint		type;			/* 0 - Normal stacked bar, 1 - proc special */
+	GList		*cd_list;		/* Same cd_list as in ChartConfig struct */
+	gint		cd_list_index;
+	void		(*draw_chart)(gpointer);
+	gpointer	draw_chart_data;
+
+	gint		x, y, w, h;		/* h tracks h in ChartConfig	*/
+
+	ChartConfig	*config;
+	gboolean	shown;
+	gboolean	redraw_all;
+
+	gint		position,
+				tail;
+	gboolean	primed;
+
+	gint		privat;
 	gint		i0;				/* Draw start index */
-	gint		*pDataOut,
-				*pDataIn;
 	gint		alloc_width;
 
-	gint		in_maxval,		/* To get data enclosing rectangle to 	*/
-				out_maxval;		/*  optimize chart stencil draws.		*/
-	gint		maxval;
-
 	gint		scale_max;
-	gint		scale_min;
-	gulong		prevOut,
-				prevIn,
-				prevTotal;
-	gint		position;
-	gint		primed;
+	gint		maxval,
+				maxval_auto,
+				maxval_auto_base,
+				maxval_peak;
+	gulong		previous_total;
+
+	gint		style_id;
+
+	GdkPixmap	*pixmap;			/* The expose pixmap.			*/
 
-	GdkPixmap	*bg_clean_pixmap;
-	GdkBitmap	*bg_mask;
-	gint		transparency,
-				need_redraw;
-	gboolean	override_images;
 	GdkImlibImage
 				*bg_image,
-				*grid_image;
+				*bg_grid_image;
+	GdkPixmap	*bg_pixmap,			/* Working bg with grid lines	*/
+				*bg_src_pixmap,		/* bg with no grid lines		*/
+				*bg_grid_pixmap,	/* This + bg_src_pixmap builds bg_pixmap*/
+				*bg_clean_pixmap;	/* This + trans builds bg_src_pixmap */
+	GdkBitmap	*bg_mask;
+	gint		transparency;
+	gboolean	bg_image_override;
+
+	GtkWidget	*config_window;
+	gpointer	*monitor;
 	}
 	Chart;
 
 
+  /* Values for ChartData draw_style */
+#define	CHARTDATA_IMPULSE	0
+#define	CHARTDATA_LINE		1
+
+  /* Values for ChartData flags */
+#define CHARTDATA_ALLOW_HIDE			1
+#define CHARTDATA_NO_CONFIG_DRAW_STYLE	2
+#define CHARTDATA_NO_CONFIG_INVERTED	4
+#define CHARTDATA_NO_CONFIG_SPLIT		8
+#define CHARTDATA_NO_CONFIG				0xf
 
 typedef struct
 	{
-	Panel	*p;
-	Decal	*d;
-	void	(*cb_button_event)();
-	gint	(*cb_in_button)();
-	void	*data;
-	gint	cur_index;
-	gint	pressed_index;
-	gint	saved_index;
-	gint	sensitive;
-	gint	type;
-	gint	id_press,
-			id_release,
-			id_enter,
-			id_leave;
+	gint		flags;
+
+	Chart		*chart;			/* Chart this data belong to */
+	gchar		*label;
+	gulong		current,
+				previous,
+				discard;
+	gboolean	monotonic;
+	ChartLayer	layer;			/* The image + grid for this data layer */
+	GdkBitmap	*data_bitmap;	/* Draw data here, use as clipmask for layer*/
+	gint		*data;
+
+	gint		draw_style;
+	gboolean	inverted;
+	gboolean	hide;
+
+	gboolean	split_chart;
+	gfloat		split_fraction,
+				split_share;
+	GtkWidget	*split_fraction_spin_button;
+
+	gint		y_p,
+				y_pp;
+	gint		y,				/* Each data layer draws at an offset y and */
+				h,				/* within height h of its parent Chart	*/
+				w;				/* Width of data allocated.				*/
+	gint		maxval;
+	}
+	ChartData;
+
+
+typedef struct
+	{
+	Panel		*p;
+	Decal		*d;
+	void		(*cb_button_click)();
+	gint		(*cb_in_button)();
+	gpointer	data;
+	gint		cur_index;
+	gint		pressed_index;
+	gint		saved_index;
+	gint		sensitive;
+	gint		type;
+	gint		id_press,
+				id_release,
+				id_enter,
+				id_leave;
+	void		(*cb_button_right_click)();
+	gpointer	right_data;
 	}
 	DecalButton;
 
@@ -498,6 +653,93 @@ typedef struct
 	Launcher;
 
 
+/* ------- Alerts ------- */
+#define	GKRELLM_ALERTCONFIG_KEYWORD	"alert_config"
+
+typedef struct
+	{
+	gboolean	warn_on,
+				alarm_on;
+	gfloat		warn_limit,
+				alarm_limit;
+	GtkWidget	*warn_limit_spin_button,
+				*alarm_limit_spin_button;
+	}
+	Trigger;
+
+typedef struct
+	{
+	Decal		*decal;
+	gint		x, y, w, h;
+	GdkPixmap	*pixmap;
+	GdkBitmap	*mask;
+	gint		frame,
+				nframes,
+				dir;
+	}
+	AlertDecal;
+
+typedef struct
+	{
+	Krell	*krell;
+	gint	krell_position;
+	}
+	AlertKrell;
+
+typedef struct
+	{
+	Panel		*panel;
+	gchar		*name,
+				*unit_string;
+	AlertDecal	ad;
+	AlertKrell	ak;
+
+	gboolean	activated,
+				freeze,
+				do_panel_updates,
+				suppress_command,
+				check_low,
+				check_high,
+				check_boolean;
+	gchar		*warn_command,
+				*alarm_command,
+				*voice_string;
+	gint		warn_repeat_set,
+				warn_repeat,
+				alarm_repeat_set,
+				alarm_repeat,
+				voice_repeat_set,
+				voice_repeat;
+
+	void		(*cb_trigger)();
+	gpointer	cb_trigger_data;
+	void		(*cb_stop)();
+	gpointer	cb_stop_data;
+	void		(*cb_config)();
+	gpointer	cb_config_data;
+
+	GtkWidget	*config_window,
+				*warn_command_entry,
+				*alarm_command_entry,
+				*voice_string_entry,
+				*warn_repeat_spin_button,
+				*alarm_repeat_spin_button,
+				*voice_repeat_spin_button;
+
+	gfloat		max_high,		/* Adjustment values */
+				min_low,
+				step0,
+				step1;
+	gint		digits;
+
+	Trigger		low,
+				high;
+	}
+	Alert;
+
+/* ------------------------ */
+
+
 /* ================= Theme Config ==================*/
 
 #define	CHART_PANEL_TYPE	0
@@ -523,7 +765,7 @@ typedef struct
 #define	INET_STYLE_NAME			"inet"
 
 #define	MEM_STYLE_NAME			"mem"
-#define	SWAP_STYLE_NAME			"swap"
+#define SWAP_STYLE_NAME			"swap"
 #define	FS_STYLE_NAME			"fs"
 #define	MAIL_STYLE_NAME			"mail"
 #define	APM_STYLE_NAME			"apm"
@@ -558,6 +800,7 @@ struct	ThemeConfig
 	gint		cpu_sys_activity;
 	gint		sensor_temp_files;
 
+	gboolean	initialized;
 	gint		max_chart_height;
 	gint		monitor_height;
 
@@ -577,10 +820,22 @@ struct	ThemeConfig
 				frame_bottom_height;
 	gint		chart_width_ref;
 
+	gint		chart_history_length;
 	gint		chart_height_min;
 	gint		chart_height_max;
 	gint		allow_scaling;
 	gboolean	config_modified;
+	gboolean	any_transparency;
+
+	gint		update_HZ;
+	gint		chart_width;
+	gboolean	save_position;
+	gboolean	large_info_font;
+	gboolean	track_gtk_rcfiles;
+
+	gboolean	enable_hostname,		/* No separate hostname config. */
+				hostname_short,
+				enable_system_name;
 
 	gint		rx_led_x;				/* Move these to net monitor */
 	gint		rx_led_y;
@@ -618,23 +873,16 @@ struct	ThemeConfig
 				frame_left_border,
 				frame_right_border,
 				button_panel_border,
-				button_meter_border,
-				bg_timer_border;
+				button_meter_border;
 
 	GdkImlibImage
 				*decal_misc_image;
 	GdkPixmap	*decal_misc_pixmap;
 	GdkBitmap	*decal_misc_mask;
 
-	/* The next 4 images should be local to their monitor file (they are
-	|  extra local images) but cannot because of 0.9.x theme compatibility.
+	/* These two vars should be handled in mail.c, but for historical reasons
+	|  gkrellmrcs define them in a way that must be handled in config.c
 	*/
-	GdkImlibImage
-				*decal_net_led_image,
-				*decal_timer_button_image,
-				*decal_mail_image,
-				*bg_timer_image;
-
 	gint		decal_mail_frames,
 				decal_mail_delay;
 
@@ -656,15 +904,23 @@ struct	ThemeConfig
 				*normal_font_string,
 				*small_font_string;
 
-	GdkImlibImage
-				*data_in_image,
-				*data_out_image,
-				*data_in_grid_image,
-				*data_out_grid_image;
+	GdkImlibImage *decal_alarm_image;
+	GdkImlibImage *decal_warn_image;
+	gint		decal_alarm_frames,
+				decal_warn_frames;
 
+	GdkImlibImage *bg_separator_image;
+	GdkPixmap	*bg_separator_pixmap;
+	gint		bg_separator_height;
+
+	GdkImlibImage *data_in_image,		/* Default data layers 0,2, ... */
+				*data_in_grid_image;
 	GdkPixmap	*data_in_pixmap,
-				*data_out_pixmap,
-				*data_in_grid_pixmap,
+				*data_out_pixmap;
+
+	GdkImlibImage *data_out_image,		/* Default data layers 1,3, ... */
+				*data_out_grid_image;
+	GdkPixmap	*data_in_grid_pixmap,
 				*data_out_grid_pixmap;
 
 	gchar		*chart_in_color,
@@ -682,12 +938,14 @@ struct	ThemeConfig
 	gint		bg_grid_mode;
 	gint		theme_n_alternatives;
 	gint		theme_alternative;
-	gint		font_override;
+	gboolean	font_override,
+				use_fontset;
 	gint		m2;
 	gint		use_top_bottom_margins;
 
 	GList		*chart_name_list,		/* Move these lists to monitor_list */
 				*meter_name_list,
+				*custom_name_list,
 				*bg_chart_image_list,
 				*bg_grid_image_list,
 				*bg_panel_image_list,
@@ -696,30 +954,20 @@ struct	ThemeConfig
 				*krell_meter_image_list,
 				*chart_style_list,
 				*panel_style_list,
-				*meter_style_list;
+				*meter_style_list,
+				*custom_style_list;
 	};
 
   /* Debugs for debug_level	*/
-#define	DEBUG_MISC		0x1
-#define	DEBUG_IMAGES	0x2
-#define	DEBUG_GKRELLMRC	0x4
-#define	DEBUG_STYLES	0x8
 #define DEBUG_MAIL		0x10
 #define DEBUG_NET		0x20
 #define DEBUG_TIMER		0x40
-#define	DEBUG_SIZES		0x80
-#define	DEBUG_PLUGIN	0x100
-#define	DEBUG_SENSORS	0x200
-
-/* Internal debugging, not doc'ed in gkrellm --help */
-#define DEBUG_KRELL		0x10000
+#define	DEBUG_SENSORS	0x80
 
 
 extern struct ThemeConfig  GK;
 
 
-
-
 /* ================= User Config ==================*/
 
   /* Values for Monitor id
@@ -744,6 +992,7 @@ extern struct ThemeConfig  GK;
 #define	MON_HOST	13
 #define	MON_SWAP	14
 #define MON_VOLTAGE	15
+#define N_BUILTIN_MONITORS	16
 #define MON_PLUGIN	16
 
 #define	MON_ID_MASK		0x1f
@@ -755,49 +1004,35 @@ extern struct ThemeConfig  GK;
 #define	GRAVITY(n)			((n) << 12)
 #define	PLUGIN_GRAVITY(m)	(((m)->insert_before_id & MON_GRAVITY_MASK) >> 12)
 
-#define	INSERT_BEFORE_ID(p)		((p)->insert_before_id & MON_ID_MASK)
-#define INSERT_AFTER_FLAG(p)	((p)->insert_before_id & MON_INSERT_AFTER)
+#define	PLUGIN_INSERT_BEFORE_ID(p)	((p)->insert_before_id & MON_ID_MASK)
+#define PLUGIN_INSERT_AFTER(p)	\
+							(((p)->insert_before_id & MON_INSERT_AFTER) >> 9)
 
 #define	MONITOR_ID(m)			((m)->id & MON_ID_MASK)
 #define	MONITOR_CONFIG(m,flag)	(((m)->id & MON_CONFIG_MASK) & flag)
 
 
-struct UserConfig
-	{
-	gint	enable_hostname;
-	gint	hostname_short;
-
-	gint	save_position;
-
-	gint	fixed_scale;			/* 0 => autoscale, or 1-5 fixed grids */
-	gint	update_HZ;
-	gint	spare;
-
-	gint	chart_width;
-	gint	chart_height[N_CHART_MONITORS];
-
-	gint	proc_load_bar_graph,
-			proc_clip_processes;
-
-	gint	ema_period;
-	gint	bump_HZ;
-	gint	large_info_font;
-	gint	track_gtk_rcfiles;
-	};
-
-extern struct UserConfig  UC;
-
 
 typedef struct
 	{
-	GtkWidget		*spacer_top_vbox,
+	GtkWidget		*main_vbox,
+					*spacer_top_vbox,
 					*vbox,
 					*spacer_bottom_vbox,
 					*notebook_config_widget;
 	gint			notebook_config_page;
 	GtkCTreeNode	*ctree_config_node;
-	gint			enabled,
-					created;
+	void			(*cb_disable_plugin)(void);
+	gint			insert_before_id,
+					gravity,
+					button_id;
+	gboolean		insert_after;
+	gboolean		enabled,
+					created,
+					from_command_line,
+					spacers_shown;
+	Style			*panel_style,
+					*chart_style;
 	gchar			*style_name;
 	gint			style_type,
 					style_id;	/* helper until I consolidate image lists */
@@ -831,7 +1066,7 @@ typedef struct
 	
 	void		(*undef2)(void);
 	void		(*undef1)(void);
-	MonPrivate	*private;
+	MonPrivate	*privat;
 
 	gint		insert_before_id;		/* If plugin, insert before this mon*/
 	
